require(boot)

cata.phases.filter <- function(xv,yv, drop.thresh=0.2, pause.dt=0.1,...) {
  inds <- order(xv)
  xv <- xv[inds]
  yv <- yv[inds]
  n <- length(xv)

  max.y <- yv[1]
  max.ind <- 1

  min.y <- yv[1]
  min.ind <- 1

  phase.growth <- 1
  phase.shortening <- 2
  phase.paused <- 3

  cata <- data.frame(phase=phase.growth,start=1,end=NA)
  phase <- phase.growth
  for (i in 2:n) {
    if (phase == phase.growth) {
      if (yv[i] > max.y) {
        max.y <- yv[i]
        max.ind <- i
      } else {
        dy <- max.y - yv[i]
        if (dy > drop.thresh) {
          cata$end[nrow(cata)] <- max.ind

          if (xv[i-1] - xv[max.ind] > pause.dt) {
            cata <- rbind(cata, data.frame(phase=phase.paused, start=max.ind+1,end=i-1))
            max.ind <- i-1
          }

          phase <- phase.shortening
          cata <- rbind(cata, data.frame(phase=phase.shortening, start=max.ind+1,end=NA))

          min.ind <- i
          min.y <- yv[i]
        }
      }
    } else if (phase == phase.shortening) {
      if (yv[i] < min.y) {
        min.y <- yv[i]
        min.ind <- i
      } else {
        dy <- yv[i] - min.y 
        if (dy > drop.thresh) {
          cata$end[nrow(cata)] <- min.ind
          phase <- phase.growth
          cata <- rbind(cata, data.frame(phase=phase.growth, start=min.ind+1,end=NA))

          max.ind <- i
          max.y <- yv[i]
        }
      }
    }
  }

  cata$end[nrow(cata)] <- length(yv)
  cata$t.start <- xv[cata$start]
  cata$t.end   <- xv[cata$end]
  cata$dt      <- cata$t.end - cata$t.start
  cata$delta   <- yv[cata$end] - yv[cata$start]
  cata$which   <- 0
  cata$which[cata$phase == phase.growth    ] =  1
  cata$which[cata$phase == phase.shortening] = -1
  cata$which[cata$phase == phase.paused] = 0

  cata
}

cata.growth.shortening.rates <-
  function(xv,yv,comb.phases) {
    comb.phases <- comb.phases[order(comb.phases$start),]
    n <- nrow(comb.phases)

    phase.rates <- data.frame(rate=0, phases=comb.phases$which)
    for (i in 1:n) {
      i0 <- comb.phases$start[i]
      i1 <- comb.phases$end[i]
      fit <- lm(y~x, data.frame(x=xv[i0:i1], y=yv[i0:i1]))
      phase.rates$rate[i] <- coef(fit)[[2]]
    }

    phase.rates
  }

cata.count.events <- function(xv,yv,...) {
  comb.phases <- cata.phases.filter(xv,yv, ...)

  opts <- list(...)
  if (!is.null(opts$min.cata.time)) {
    comb.phases <- subset(comb.phases, t.end >= opts$min.cata.time)
  }

  min.len = opts$min.length
  if (is.null(min.len)) {
    min.len <- 0.0
  }

  num.cata <- 0
  num.resc <- 0

  growth.time  <- 0
  shorten.time <- 0

  growth.dist  <- 0
  shorten.dist <- 0

  first.cata.time <- Inf

  df <- data.frame()

  if (comb.phases$which[1] != -1) {
    in.cata <- FALSE
    for (i in 1:nrow(comb.phases)) {
      wh <- comb.phases$which[i]
      dt <- comb.phases$dt[i]
      delta <- comb.phases$delta[i]
      start.len   <- yv[comb.phases$start[i]]
      end.len <- yv[comb.phases$end[i]]
      if (!in.cata) {
        if (wh == -1 && start.len >= min.len) {
          if (first.cata.time > comb.phases$t.start[i]) {
            first.cata.time <- comb.phases$t.start[i]
          }
          in.cata <- TRUE
          num.cata <- num.cata + 1
        }
      } else {
        if (wh == 1) {
          in.cata <- FALSE
          num.resc <- num.resc + 1
        }
      }

      if (!in.cata) {
        growth.time <- growth.time + dt
        growth.dist <- growth.dist + delta
      } else {
        shorten.time <- shorten.time + dt
        shorten.dist <- shorten.dist + delta
      }
    }
  } else {
    first.cata.time <- 0
  }

  data.frame(
    num.resc = num.resc,
    num.cata = num.cata,
    first.cata.time = first.cata.time,
    growth.time = growth.time,
    shorten.time = shorten.time,
    growth.dist = growth.dist,
    shorten.dist = shorten.dist)
}

open.maybe.compressed <- function(base.name) {
  file.ext = c( '.xz', '.bz2', '.gz' )
  open.fun  = c( xzfile, bzfile, gzfile )
  ext.names <- paste(base.name,file.ext,sep='')
  wh <- file.exists(ext.names)
  f <- open.fun[wh]
  if (length(f) < 1) {
    return(file(base.name))
  }

  n <- ext.names[wh]
  return(f[[1]](n))
}

create.tippos <- function(prefix) {
  within(list(), {
    sim.prefix <- prefix
    sim.path     <- sprintf('%s', prefix)
    tippos.small.file <- sprintf('%s/%s_tippos_small.txt', sim.path,prefix)
    param.file   <- sprintf('%s/%s_params.txt', sim.path,prefix)

    data.tippos <- read.table(file=open.maybe.compressed(tippos.small.file), header=T)
    colnames(data.tippos)[[1]] <- 'task'

    data.tippos$mt.len.3 <- 8e-3*(data.tippos$plus_avg - data.tippos$minus_top3)
    data.tippos$mt.len.5 <- 8e-3*(data.tippos$plus_avg - data.tippos$minus_top5)
    data.tippos$mt.len.7 <- 8e-3*(data.tippos$plus_avg - data.tippos$minus_top7)

    data.params <- read.table(file=open.maybe.compressed(param.file), header=T)
    data.params.sets <- subset(ddply(data.params,.(set), function(df) { df[1,] }),
                               select=c(-task))
    data.tippos.comb <- merge(data.params,data.tippos,by='task')

    max.times <- ddply(data.tippos, .(task), summarize, max.time.secs=1e-5*max(time))
  })
}

add.cata.counts <- function(lst,...) {
  within(lst, {
    new.cata.counts <- ddply(
      data.tippos.comb, .(task,set), 
      function(df) { 
        cata.count.events(df$time*1e-5,df$mt.len.7,...) 
      })

    new.cata.counts.set <- merge(
      data.params.sets,
      ddply(new.cata.counts, .(set),
            function(df) { 
              total.growth.time  <- sum(df$growth.time)
              total.shorten.time <- sum(df$shorten.time)
              total.growth.dist  <- sum(df$growth.dist)
              total.shorten.dist <- sum(df$shorten.dist)
              total.num.cata <- sum(df$num.cata)
              total.num.resc <- sum(df$num.resc)
              data.frame(
                total.growth.time=total.growth.time, 
                total.shorten.time=total.shorten.time, 
                total.num.cata=total.num.cata, 
                total.num.resc=total.num.resc, 
                cata.freq=total.num.cata/total.growth.time, 
                resc.freq=total.num.resc/max(total.shorten.time,1),
                cata.per.dist=total.num.cata/total.growth.dist,
                resc.per.dist=total.num.resc/total.shorten.dist)
            }),
      by='set')
  })
}

calc.cata.freq <- function(cata.data) {
  total.growth.time  <- sum(cata.data$growth.time)
  total.num.cata <- sum(cata.data$num.cata)
  data.frame(total.num.cata=total.num.cata,
             total.growth.time=total.growth.time,
             cata.freq=total.num.cata/total.growth.time)
}

cata.count.aggregate <- function(lst, calc.ci=FALSE) {
  cata.data.by.set <- ddply(
    lst$new.cata.counts, .(set),
    function(df) {
      if (!calc.ci) {
        total.growth.time  <- sum(df$growth.time)
        total.shorten.time <- sum(df$shorten.time)
        total.growth.dist  <- sum(df$growth.dist)
        total.shorten.dist <- sum(df$shorten.dist)
        total.num.cata <- sum(df$num.cata)
        total.num.resc <- sum(df$num.resc)
        data.frame(
                   total.num.cata=total.num.cata,
                   total.growth.time=total.growth.time,
                   cata.freq=total.num.cata/total.growth.time)
      } else {
        b.df <- boot(df,function(df,ind) {
                       calc.cata.freq(df[ind,])$cata.freq
                     },
                     R=1000)
        ci.df <- boot.ci(b.df, type=c("perc"))
        cbind(calc.cata.freq(df),
              cata.freq.lo = ci.df$percent[4],
              cata.freq.hi = ci.df$percent[5],
              total.growth.time  = sum(df$growth.time),
              total.shorten.time = sum(df$shorten.time),
              total.growth.dist  = sum(df$growth.dist),
              total.shorten.dist = sum(df$shorten.dist),
              total.num.cata = sum(df$num.cata),
              total.num.resc = sum(df$num.resc))
      }
    })
  merge(lst$data.params.sets, cata.data.by.set, by='set')
}

growth.filter.refine <- function(xv,yv,seg.npts=20) {
  npts <- length(yv)
  nsegs <- npts

  cmat <- array(c(rep(1,npts),xv,yv), c(npts,3))
  r <- array(0, c(nsegs,7))
  for (i in 1:npts) {
    i0 <- max(1,i - seg.npts)
    i1 <- min(npts,i + seg.npts)
    n <- i1-i0+1

    seg.fit.1 <- lm.fit(as.matrix(cmat[i0:i1,1:2]),cmat[i0:i1,3])
    seg.fit.2 <- lm.fit(as.matrix(cmat[i0:i1,1]),cmat[i0:i1,3])

    r[i,1] <- seg.fit.1$coefficients[1]
    r[i,2] <- seg.fit.1$coefficients[2]
    r[i,3] <- seg.fit.2$coefficients[1]
    r[i,4] <- sum(seg.fit.1$residuals^2)
    r[i,5] <- sum(seg.fit.2$residuals^2)
    r[i,6] <- seg.fit.1$df.residual
  }

  r[,7] <- (r[,5] - r[,4])/n
  data.frame(t=xv,
             t.x=xv,
             y=yv,
             intercept=r[,1],
             slope=r[,2],
             mean=r[,3],
             resid1=r[,4],
             resid2=r[,5],
             avg.resid.diff=r[,7],
             df1=r[,6])
}

filter.growth <- function(xv,yv, ...) {
  first.pass <- cata.phases.filter(xv,yv,...)

  which <- array(0, length(xv))

  second.pass <- NULL
  for (i in 1:nrow(first.pass)) {
    i0 <- first.pass$start[i]
    i1 <- first.pass$end[i]

    if (first.pass$which[i] != 1) {
      which[i0:i1] <- first.pass$which[i]
      next
    }

    p2.result <- cata.phases.filter(xv[i0:i1],yv[i0:i1], drop.thresh=0.05)
    p2.result$start <- p2.result$start + i0 - 1
    p2.result$end   <- p2.result$end   + i0 - 1
    second.pass <- rbind(second.pass,p2.result)
  }

  if (length(which) != length(xv)) {
    cat("which! 1\n")
    browser()
  }

  third.pass <- NULL
  for (i in 1:nrow(second.pass)) {
    i0 <- second.pass$start[i]
    i1 <- second.pass$end[i]

    if (second.pass$which[i] != 1) {
      which[i0:i1] <- second.pass$which[i]
      next
    }

    i0 <- second.pass$start[i]
    i1 <- second.pass$end[i]

    p3.result <- growth.filter.refine(xv[i0:i1],yv[i0:i1], seg.npts=10)
    pause.phases <- rle(p3.result$avg.resid.diff <= 1e-5)

    ph.which  <- pause.phases$values
    ph.len <- pause.phases$lengths

    ph.which.0 <- pause.phases$values
    ph.len.0 <- pause.phases$lengths

    min.run.len <- 10
    npass <- 0
    repeat {
      npass <- npass + 1
      nrun <- length(ph.len)
      min.ind <- which.min(ph.len)
      min.run <- ph.len[min.ind]
      if ((min.run >= 10) || (length(ph.len) == 1)) {
        break
      }

      if (min.ind == 1) {
        ph.len[2] <- ph.len[2] + ph.len[1]
        ph.len   <- ph.len[-1]
        ph.which <- ph.which[-1]
      } else if (min.ind == nrun) {
        ph.len[nrun-1] <- ph.len[nrun-1] + ph.len[nrun]
        ph.len   <- ph.len[-nrun]
        ph.which <- ph.which[-nrun]
      } else {
        ph.len[min.ind-1] <- ph.len[min.ind-1] + ph.len[min.ind] + ph.len[min.ind+1]
        ph.len <- ph.len[c(-min.ind,-(min.ind+1))]
        ph.which <- ph.which[c(-min.ind,-(min.ind+1))]
      }

      tryCatch(local({
        if (sum(ph.len) != sum(ph.len.0)) {
          cat("lengths do not agree\n")
          browser()
        }
      }), error=function(...) {
        cat('ph.len   ',ph.len,"\n")
        cat('ph.len.0 ',ph.len.0,"\n")
        browser()
      })

      if (sum(ph.len) + (i0-1) > length(xv)) {
        cat("--\n")
        browser()
      }
    }

    ph.ends   <- cumsum(ph.len)
    if (length(ph.ends) > 1) {
      ph.starts <- c(1, ph.ends[1:(length(ph.ends)-1)]+1)
    } else {
      ph.starts <- 1
    }
    ph.ends   <- ph.ends + (i0-1)
    ph.starts <- ph.starts + (i0-1)

    if (any(ph.starts > length(xv)) || (any(ph.ends) > length(xv))) {
      cat(">>\n")
      browser()
    }

    if (sum(ph.len) + (i0-1) > length(xv)) {
      cat("--\n")
      browser()
    }

    third.pass <- rbind(third.pass,
                        data.frame(n=ph.len,
                                   paused=ph.which,
                                   start=ph.starts,
                                   end=ph.ends))
  }

  for (i in 1:nrow(third.pass)) {
    i0 <- third.pass$start[i]
    i1 <- third.pass$end[i]
    wh <- 0
    if (third.pass$paused[i] == FALSE) {
      wh <- 1
    }
    which[i0:i1] <- wh
    if (length(which) != length(xv)) {
      cat("which! pre-2", i, "\n")
      browser()
    }
  }

  if (length(which) != length(xv)) {
    cat("which! 2\n")
    browser()
  }

  all.runs <- rle(as.vector(which))
  all.ends <- cumsum(all.runs$lengths)
  if (length(all.ends) > 1) {
    all.starts <- c(1, all.ends[1:(length(all.ends)-1)]+1)
  } else {
    all.starts <- 1
  }

  t0 <- xv[all.starts]
  t1 <- xv[all.ends]
  dt <- t1-t0

  if (any(all.ends < all.starts) || any(all.starts > length(xv)) || any(all.ends > length(xv))) {
    cat("--all.ends--\n")
    browser()
  }

  all.phases <- data.frame(phase = 1:length(all.ends),
                           start = all.starts,
                           end   = all.ends,
                           t.start = t0,
                           t.end   = t1,
                           dt      = dt,
                           which   = all.runs$values)

  all.phases
}

analyze.rescue.sims <- function(tp.sims) {
  max.time <- 25
  bin.width <- 1
  num.bins <- 25/bin.width
  ddply(
    tp.sims,
    .(task), 
    function(df) {
      xv <- df$time * 1e-5
      yv <- df$mt.len.7
      ok <- !is.nan(xv) & !is.nan(yv)
      xv <- xv[ok]
      yv <- yv[ok]
      sec.bins <- trunc(xv/bin.width)
      sec.bins[sec.bins>=num.bins] <- num.bins-1
      resc.time <- Inf
      resc.dist <- Inf
      i.min <- length(xv)
      for (b in 0:(num.bins-1)) {
        wh <- (sec.bins == b)
        n <- sum(wh)
        if (n < 2) {
          next
        }
        X <- matrix(1,n,2)
        X[,2] <- xv[wh]
        fit <- lm.fit(X,yv[wh])
        fit.slope <- coef(fit)[[2]]
        if (fit.slope > -0.05) {
          i.min <- min(which(wh))
          resc.time <- xv[i.min]
          resc.dist <- yv[1] - yv[i.min]
          break
        }
      }

      Xs <- matrix(1, i.min, 2)
      Xs[,2] <- xv[1:i.min]
      shorten.fit <- lm.fit(Xs,yv[1:i.min])
      shorten.rate <- coef(shorten.fit)[[2]]

      if (is.infinite(resc.time)) {
        resc.time <- max.time
        resc.dist <- yv[1]
      }
      data.frame(resc.time = resc.time, resc.dist = resc.dist, shorten.rate=shorten.rate)
    })
}

add.rescue.analysis <- function(lst) {
  within(lst,{
    resc.analysis <- merge(
      data.params,
      analyze.rescue.sims(data.tippos))
  })
}

print.graphs <- function(gr, png.file=NULL, pdf.file=NULL, ...) {
  if (is.function(gr)) {
    pl <- gr()
  } else {
    pl <- gr
  }

  opts <- list(...)
  if (is.null(opts$png.width)) {
    opts$png.width <- 512
  }

  if (is.null(opts$png.height)) {
    opts$png.height <- opts$png.width
  }

  if (is.null(opts$pdf.width)) {
    opts$pdf.width <- 6
  }

  if (is.null(opts$pdf.height)) {
    opts$pdf.height <- opts$pdf.width
  }

  if (!is.null(png.file) &&
      substr(png.file,nchar(png.file)-3,nchar(png.file)) != '.png') {
    png.file <- sprintf("%s.png",png.file)
  }

  if (!is.null(pdf.file) &&
      substr(pdf.file,nchar(pdf.file)-3,nchar(pdf.file)) != '.pdf') {
    pdf.file <- sprintf("%s.pdf",pdf.file)
  }

  if (!is.null(pdf.file)) {
    cat(sprintf('---[ printing pdf: %s ]---\n',pdf.file))
    pdf(pdf.file, width=opts$pdf.width, height=opts$pdf.height)
    try(plot(pl))
    dev.off()
  }

  if (!is.null(png.file)) {
    cat(sprintf('---[ printing png: %s ]---\n',png.file))
    png(png.file, width=opts$png.width, height=opts$png.height)
    try(plot(pl))
    dev.off()
  }
}

