source('./Common/analysis_tip.R')
source('./Common/salmon_88_analysis.R')

require(xts)

reread.data <- FALSE
make.plots  <- TRUE

sim.name <- 'mcata7b'
plot.pfx <- 'mcata7b'

if (exists('sim.data')) {
  sim.data$plot.prefix <- plot.pfx
}

if (reread.data || !exists('sim.data') || sim.data$sim.prefix != sim.name) {
  sim.data <- create.tippos(sim.name)

  sim.data$plot.prefix <- plot.pfx
}

sim.data$data.tippos.maxlen <- ddply(sim.data$data.tippos, .(task), summarize, max.len=max(mt.len.7))
sim.data$filtered.tasks <- sim.data$data.tippos.maxlen$task[ sim.data$data.tippos.maxlen$max.len >= 0.75 ]

sim.data$data.tippos.filt <- subset(sim.data$data.tippos, task %in% sim.data$filtered.tasks)

sim.data$growth.rates.by.phase <- 
  merge(
    sim.data$data.params,
    ddply(sim.data$data.tippos.filt, .(task),
          function(df) {
            xv <- 1e-5*df$time
            yv <- df$mt.len.7
            inds <- order(xv)
            xv <- xv[inds]
            yv <- yv[inds]

            phases <- filter.growth(xv,yv)
            rates <- cata.growth.shortening.rates(xv,yv,phases)
            rates$um.per.min <- rates$rate * 60
            rates$max.growth <- max(df$mt.len.7)
            rates
          }))

sim.data.1 <- local({
  sim.data$data.tippos.comb <- merge(sim.data$data.params, sim.data$data.tippos.filt, by='task')
  sim.data <- add.cata.counts(sim.data, min.length = 0.30, drop.thresh = 0.25)
  sim.data$cata.data.aggregate <- cata.count.aggregate(sim.data,calc.ci=T)

  sim.data
})

orig.wd <- getwd()
if (!dir.exists(plot.pfx)) {
  dir.create(plot.pfx)
}
setwd(plot.pfx)

if (make.plots) {
  try(with(sim.data, {
       print.graphs(
         ggplot(sim.data.1$cata.data.aggregate,
                aes(x=conc, y=cata.freq, color=factor(scanname))) +
           geom_point(size=4) +
           geom_line() +
           guides(color=F) +
           labs(x=expression('Concentration'~(mu*M)),
                y=expression('Catastrophe freq'~(sec^-1))) +
           geom_point(data=salmon88.fig7$minus.end,
                         aes(y=freq,x=conc), inherit.aes=F, shape=4),
         pdf.file=sprintf('%s_cata_freq.pdf', plot.prefix))

       print.graphs(
         ggplot(mutate(subset(growth.rates.by.phase,phases==1 & um.per.min > 0),
                       conc_f = factor(conc, levels=c(7.5,10.0,12.5,15.0))),
                aes(x=conc_f, y=um.per.min, color=factor(scanname))) +
           geom_boxplot() +
           guides(color=F) +
           geom_point(data=mutate(data.frame(conc=c(7.5,10.0,12.5,15.0)),
                                  conc_f = factor(conc,levels=c(7.5,10.0,12.5,15.0)),
                                  um.per.min=0.15*conc-0.85, 
                                  phases=1),shape=4, color='black'),
         pdf.file=sprintf('%s_growth_rates.pdf',plot.prefix))

  }))
}

setwd(orig.wd)
