require "tubesheet"
require "chemdriver"
require "analysis"

include ChemAnalysis

def measure_positions(driver, fsm, t)
  npf = (driver.ncol/2).to_i

  row_range = driver.find_row_range
  tube0 = fsm.tube_first_rows
  tube1 = fsm.tube_last_rows

  minus_end   = (0...npf).map {|pf| ((row_range[2*pf][0]+1)/4).to_i }.sort
  plus_end    = (0...npf).map {|pf| ((row_range[2*pf][1]+1)/4).to_i }.sort.reverse

  (plus_avg, plus_sd) = calc_mean_and_sd(plus_end)
  (minus_avg, minus_sd) = calc_mean_and_sd(minus_end)

  (minus_tube_avg, minus_tube_sd) = calc_mean_and_sd(tube0)
  (plus_tube_avg , plus_tube_sd ) = calc_mean_and_sd(tube1)

  (plus_top7, _) = calc_mean_and_sd(plus_end[0...7])
  (plus_top5, _) = calc_mean_and_sd(plus_end[0...5])
  (plus_top3, _) = calc_mean_and_sd(plus_end[0...3])

  (minus_top7, _) = calc_mean_and_sd(minus_end[0...7])
  (minus_top5, _) = calc_mean_and_sd(minus_end[0...5])
  (minus_top3, _) = calc_mean_and_sd(minus_end[0...3])

  {
    :plus_avg   => plus_avg,
    :plus_sd    => plus_sd,

    :minus_avg  => minus_avg,
    :minus_sd   => minus_sd,

    :plus_tube_avg  => plus_tube_avg,
    :minus_tube_avg => minus_tube_avg,

    :plus_top7  => plus_top7,
    :plus_top5  => plus_top5,
    :plus_top3  => plus_top3,

    :minus_top7 => minus_top7,
    :minus_top5 => minus_top5,
    :minus_top3 => minus_top3,
  }
end

def process_job(job,state_fname,only_summary)
  statef = File.open(state_fname)
  (_,nrow,ncol) = Tubesheet::ChemDriver.read_state_header(statef)
  driver = Tubesheet::ChemDriver.new(nrow,ncol)
  fsm = Tubesheet::FastSheetModel.new(driver)
  statef.seek(0)

  timepoints = []

  all_data = {}

  begin
    hdr = driver.read_state(statef)
    timepoints << hdr[:time]
    data0 = measure_positions(driver,fsm,0)
    aggregate(all_data, data0)

    t=1
    while ! statef.eof?
      hdr = driver.read_state(statef)
      break if hdr.nil?
      timepoints << hdr[:time]
      data1 = measure_positions(driver,fsm,t)
      aggregate(all_data, data1)
      t += 1
    end

    puts "###"
    colnames = all_data.keys
    puts "%5s %16s %s" % [ 'job', 'time', colnames.map{ |n| "%16s" % [n.to_s] }.join(" ") ]

    timepoints.each_with_index do |tp,i|
      print "%5d %16.10f " % [ job, tp ]
      puts colnames.map { |n| "%16.10f" % [ all_data[n][i] ] }.join(" ")
    end

    puts "==="

  ensure
    statef.close if statef
  end
end

job   = Integer(ARGV[0])
fname = ARGV[1]

process_job(job,fname,false)

